@@ -26,7 +26,7 @@ public class Accessible extends AtkObject
 		Handle hndl = getWidget(getHandle());
 		if (null == hndl)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Widget)obj;
 		return new Widget(hndl);
