@@ -10,6 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Boxed;
+import org.gnu.glib.Handle;
 
 /**
  */
@@ -43,18 +44,14 @@ public class RadioActionEntry extends Boxed {
 		free(handle);
 	}
 
-	/****************************************
-	 * BEGINNING OF JNI CODE
-	 ****************************************/
-	native static final protected int allocate();
-	native static final protected void free(int entry);
-	native static final protected void setName(int entry, String name);
-	native static final protected void setStockId(int entry, String stockId);
-	native static final protected void setLabel(int entry, String label);
-	native static final protected void setAccelerator(int entry, String accel);
-	native static final protected void setToolTip(int entry, String tooltip);
-	native static final protected void setValue(int entry, int value);
-	/****************************************
-	 * END OF JNI CODE
-	 ****************************************/
+
+	native static final protected Handle allocate();
+	native static final protected void free(Handle entry);
+	native static final protected void setName(Handle entry, String name);
+	native static final protected void setStockId(Handle entry, String stockId);
+	native static final protected void setLabel(Handle entry, String label);
+	native static final protected void setAccelerator(Handle entry, String accel);
+	native static final protected void setToolTip(Handle entry, String tooltip);
+	native static final protected void setValue(Handle entry, int value);
+
 }
