@@ -10,6 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.GObject;
+import org.gnu.glib.Handle;
 
 /**
  * ToolItem is the base class of widgets that can be added to ToolBar objects.
@@ -27,10 +28,17 @@ public class ToolItem extends Bin {
 	/**
 	 * Construct a new ToolItem object from a handle to a native widget.
 	 */
-	ToolItem(int hndl) {
+	ToolItem(Handle hndl) {
 		super(hndl);
 	}
 	
+    /**
+     * Create an empty ToolItem.
+     */
+    public ToolItem() {
+        super( gtk_tool_item_new() );
+    }
+
 	/**
 	 * Sets whether the ToolItem is to be allocated the same size as other
 	 * homogeneous items.  The effect is that all homogeneous items will have
@@ -71,7 +79,7 @@ public class ToolItem extends Bin {
 	 * Sets the {@link ToolTips} object to be used for ToolItem, the text to be 
 	 * displayed as tooltip on the item and the private text to be used. 
 	 * @see ToolTips#setTip(Widget, String, String)
-	 * @param tooltips the {@link ToolTips} object to be used
+	 * @param toolTips the {@link ToolTips} object to be used
 	 * @param tipText text to be used as tooltip text for ToolItem
 	 * @param tipPrivate text to be used as private tooltip text
 	 * @deprecated 
@@ -84,7 +92,7 @@ public class ToolItem extends Bin {
 	 * Sets the {@link ToolTips} object to be used for ToolItem, the text to be 
 	 * displayed as tooltip on the item and the private text to be used. 
 	 * @see ToolTips#setTip(Widget, String, String)
-	 * @param tooltips the {@link ToolTips} object to be used
+	 * @param toolTips the {@link ToolTips} object to be used
 	 * @param tipText text to be used as tooltip text for ToolItem
 	 * @param tipPrivate text to be used as private tooltip text
 	 */
@@ -94,7 +102,7 @@ public class ToolItem extends Bin {
 
 	/**
 	 * Sets whether ToolItem has a drag window. When <code>TRUE</code> the ToolItem 
-	 * can be used as a drag source through {@link org.gnu.gdk.DragSource}. When ToolItem 
+	 * can be used as a drag source through DragSource. When ToolItem 
 	 * has a drag window it will intercept all events, even those that would otherwise
 	 * be sent to a child of ToolItem.
 	 * @param useDragWindow whether ToolItem has a drag window
@@ -152,7 +160,7 @@ public class ToolItem extends Bin {
 	/**
 	 * Sets whether ToolItem should be considered important. The {@link ToolButton} class 
 	 * uses this property to determine whether to show or hide its label when the 
-	 * toolbar style is {@link ToolBarStyle#_BOTH_HORIZ}. The result is that only 
+	 * toolbar style is {@link ToolBarStyle#BOTH_HORIZ}. The result is that only 
 	 * tool buttons with the IS_IMPORTANT property set have labels, an effect known 
 	 * as "priority text".
 	 * @param isImportant whether the tool item should be considered important
@@ -188,25 +196,30 @@ public class ToolItem extends Bin {
 		return Orientation.intern(gtk_tool_item_get_orientation(getHandle()));
 	}
 	
-	/**
-	 * Returns the toolbar style used for ToolItem. Custom subclasses of ToolItem 
-	 * should call this function in the handler of the GtkToolItem::toolbar_reconfigured
-	 * signal to find out in what style the toolbar is displayed and change themselves 
-	 * accordingly.
-	 *
-	 * <p>Possibilities are:
-	 *
-     * <p>GTK_TOOLBAR_BOTH, meaning the tool item should show both an icon and a label, stacked vertically
-     * GTK_TOOLBAR_ICONS, meaning the toolbar shows only icons
-     * GTK_TOOLBAR_TEXT, meaning the tool item should only show text
-     * GTK_TOOLBAR_BOTH_HORIZ, meaning the tool item should show both an icon and 
-     * a label, arranged horizontally (however, note the GtkToolButton::has_text_horizontally
-     * that makes tool buttons not show labels when the toolbar style is GTK_TOOLBAR_BOTH_HORIZ. 
-	 * @return a {@link ToolBarStyle} indicating the toolbar style used for ToolItem
-	 */
-	public ToolBarStyle getToolbarStyle() {
-		return ToolBarStyle.intern(gtk_tool_item_get_toolbar_style(getHandle()));
-	}
+    /**
+     * Returns the toolbar style used for ToolItem. Custom subclasses
+     * of ToolItem should call this function in the handler of the
+     * GtkToolItem::toolbar_reconfigured signal to find out in what
+     * style the toolbar is displayed and change themselves
+     * accordingly.
+     *
+     * <p>Possibilities are:
+     * <ul>
+     * <li>GTK_TOOLBAR_BOTH, meaning the tool item should show both an icon and a label, stacked vertically.</li>
+     * <li>GTK_TOOLBAR_ICONS, meaning the toolbar shows only icons.</li>
+     * <li>GTK_TOOLBAR_TEXT, meaning the tool item should only show text.</li>
+     * <li>GTK_TOOLBAR_BOTH_HORIZ, meaning the tool item should show
+     * both an icon and a label, arranged horizontally (however, note
+     * the GtkToolButton::has_text_horizontally that makes tool
+     * buttons not show labels when the toolbar style is
+     * GTK_TOOLBAR_BOTH_HORIZ.</li>
+     * </ul>
+     * @return a {@link ToolBarStyle} indicating the toolbar style
+     * used for ToolItem.
+     */
+    public ToolBarStyle getToolbarStyle() {
+        return ToolBarStyle.intern(gtk_tool_item_get_toolbar_style(getHandle()));
+    }
 	
 	/**
 	 * Returns the relief style of ToolItem. See {@link Button#setRelief(ReliefStyle)}. 
@@ -219,13 +232,17 @@ public class ToolItem extends Bin {
 	}
 	
 	/**
-	 * Returns the {@link MenuItem} that was last set by {@link ToolItem#setProxyMenuItem(String, MenuItem)}
-	 * i.e. the {@link MenuItem} that is going to appear in the overflow menu.
-	 * @return the {@link MenuItem} that is going to appear in the overflow menu for tool_item
+	 * Returns the {@link MenuItem} that was last set by {@link
+	 * ToolItem#setProxyMenuItem(String, MenuItem)} (ie the
+	 * {@link MenuItem} that is going to appear in the overflow
+	 * menu).
+         *
+	 * @return the {@link MenuItem} that is going to appear in the
+	 * overflow menu for tool_item.
 	 */
 	public MenuItem retrieveProxyMenuItem() {
-		int hndl = gtk_tool_item_retrieve_proxy_menu_item(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_tool_item_retrieve_proxy_menu_item(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj) 
 			return (MenuItem)obj;
 		return new MenuItem(hndl);
@@ -243,8 +260,8 @@ public class ToolItem extends Bin {
 	 * if the menuItemId matches
 	 */
 	public MenuItem getProxyMenuItem(String menuItemId) {
-		int hndl = gtk_tool_item_get_proxy_menu_item(getHandle(), menuItemId);
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_tool_item_get_proxy_menu_item(getHandle(), menuItemId);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj) 
 			return (MenuItem)obj;
 		return new MenuItem(hndl);
@@ -261,32 +278,39 @@ public class ToolItem extends Bin {
 		gtk_tool_item_set_proxy_menu_item(getHandle(), menuItemId, menuItem.getHandle());
 	}
 	
-	/****************************************
-	 * BEGINNING OF JNI CODE
-	 ****************************************/
+    /**
+     * Calling this function signals to the toolbar that the overflow
+     * menu item for this ToolItem has changed. If the overflow menu
+     * is visible when this function is called, the menu will be
+     * rebuilt.  
+     */
+    public void rebuildMenu() {
+        gtk_tool_item_rebuild_menu( getHandle() );
+    }
+
 	native static final protected int gtk_tool_item_get_type();
-	native static final protected int gtk_tool_item_new();
-	native static final protected void gtk_tool_item_set_homogeneous(int toolitem, boolean homogeneous);
-	native static final protected boolean gtk_tool_item_get_homogeneous(int toolitem);
-	native static final protected void gtk_tool_item_set_expand(int toolitem, boolean expand);
-	native static final protected boolean gtk_tool_item_get_expand(int toolitem);
-	native static final protected void gtk_tool_item_set_tooltip(int toolitem, int tooltips, String tipText, String tipPrivate);
-	native static final protected void gtk_tool_item_set_use_drag_window(int toolitem, boolean useDragWindow);
-	native static final protected boolean gtk_tool_item_get_use_drag_window(int toolitem);	
-	native static final protected void gtk_tool_item_set_visible_horizontal(int toolitem, boolean visibleHorizontal);
-	native static final protected boolean gtk_tool_item_get_visible_horizontal(int toolitem);
-	native static final protected void gtk_tool_item_set_visible_vertical(int toolitem, boolean visibleVertical);
-	native static final protected boolean gtk_tool_item_get_visible_vertical(int toolitem);
-	native static final protected boolean gtk_tool_item_get_is_important(int toolitem);
-	native static final protected void gtk_tool_item_set_is_important(int toolitem, boolean isImportant);
-	native static final protected int gtk_tool_item_get_icon_size(int toolitem);
-	native static final protected int gtk_tool_item_get_orientation(int toolitem);
-	native static final protected int gtk_tool_item_get_toolbar_style(int toolitem);
-	native static final protected int gtk_tool_item_get_relief_style(int toolitem);
-	native static final protected int gtk_tool_item_retrieve_proxy_menu_item(int toolitem);	
-	native static final protected int gtk_tool_item_get_proxy_menu_item(int toolitem, String menuItemId);
-	native static final protected void gtk_tool_item_set_proxy_menu_item(int toolitem, String menuItemId, int menuItem);
-	/****************************************
-	 * END OF JNI CODE
-	 ****************************************/
+	native static final protected Handle gtk_tool_item_new();
+	native static final protected void gtk_tool_item_set_homogeneous(Handle toolitem, boolean homogeneous);
+	native static final protected boolean gtk_tool_item_get_homogeneous(Handle toolitem);
+	native static final protected void gtk_tool_item_set_expand(Handle toolitem, boolean expand);
+	native static final protected boolean gtk_tool_item_get_expand(Handle toolitem);
+	native static final protected void gtk_tool_item_set_tooltip(Handle toolitem, Handle tooltips, String tipText, String tipPrivate);
+	native static final protected void gtk_tool_item_set_use_drag_window(Handle toolitem, boolean useDragWindow);
+	native static final protected boolean gtk_tool_item_get_use_drag_window(Handle toolitem);	
+	native static final protected void gtk_tool_item_set_visible_horizontal(Handle toolitem, boolean visibleHorizontal);
+	native static final protected boolean gtk_tool_item_get_visible_horizontal(Handle toolitem);
+	native static final protected void gtk_tool_item_set_visible_vertical(Handle toolitem, boolean visibleVertical);
+	native static final protected boolean gtk_tool_item_get_visible_vertical(Handle toolitem);
+	native static final protected boolean gtk_tool_item_get_is_important(Handle toolitem);
+	native static final protected void gtk_tool_item_set_is_important(Handle toolitem, boolean isImportant);
+	native static final protected int gtk_tool_item_get_icon_size(Handle toolitem);
+	native static final protected int gtk_tool_item_get_orientation(Handle toolitem);
+	native static final protected int gtk_tool_item_get_toolbar_style(Handle toolitem);
+	native static final protected int gtk_tool_item_get_relief_style(Handle toolitem);
+	native static final protected Handle gtk_tool_item_retrieve_proxy_menu_item(Handle toolitem);	
+	native static final protected Handle gtk_tool_item_get_proxy_menu_item(Handle toolitem, String menuItemId);
+	native static final protected void gtk_tool_item_set_proxy_menu_item(Handle toolitem, String menuItemId, Handle menuItem);
+    // GTK 2.6 additions.
+    native static final private void gtk_tool_item_rebuild_menu(Handle tool_item);
+
 }
