@@ -12,9 +12,10 @@ package org.gnu.gtk;
 import java.util.Vector;
 
 import org.gnu.glib.EventMap;
+import org.gnu.glib.EventType;
 import org.gnu.glib.GObject;
+import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
-import org.gnu.gtk.event.GtkEventType;
 import org.gnu.gtk.event.TreeViewEvent;
 import org.gnu.gtk.event.TreeViewListener;
 
@@ -95,7 +96,7 @@ import org.gnu.gtk.event.TreeViewListener;
  * </dl>
  * <p>The CellRenderer's need data to be able to display. This is set using the
  * {@link TreeViewColumn#addAttributeMapping(CellRenderer,
- * CellRendererAttribute, int)}. The renderer attributes vary with each
+ * CellRendererAttribute, DataColumn)}. The renderer attributes vary with each
  * renderer, for example CellRendererText has a TEXT attribute for the text the
  * be displayed. The final parameter is for the dataBlock in the store in which
  * the data is contained.
@@ -108,6 +109,8 @@ import org.gnu.gtk.event.TreeViewListener;
  */
 public class TreeView extends Container {
 
+    private TreeViewRowSeparatorMethod separatorFunc = null;
+
 	/**
 	 * Create a new TreeView object.
 	 */
@@ -118,7 +121,7 @@ public class TreeView extends Container {
 	/**
 	 * Creates a new tree view from a native handle
 	 */
-	public TreeView(int handle) {
+	public TreeView(Handle handle) {
 		super(handle);
 	}
 
@@ -133,8 +136,8 @@ public class TreeView extends Container {
 	 * Returns the model associated with this tree.
 	 */
 	public TreeModel getModel() {
-		int hndl = gtk_tree_view_get_model(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_tree_view_get_model(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (TreeModel)obj;
 		return new TreeModel(hndl);
@@ -174,8 +177,8 @@ public class TreeView extends Container {
 	 * Gets the TreeSelection associated with this widget
 	 */
 	public TreeSelection getSelection() {
-		int hndl = gtk_tree_view_get_selection(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_tree_view_get_selection(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (TreeSelection)obj;
 		return new TreeSelection(hndl);
@@ -186,8 +189,8 @@ public class TreeView extends Container {
 	 * @return A GtkAdjustment object, or NULL if none is currently being used.
 	 */
 	public Adjustment getHAdjustment() {
-		int hndl = gtk_tree_view_get_hadjustment(getHandle());
-		GObject obj = retrieveGObject(hndl);
+	    Handle hndl = gtk_tree_view_get_hadjustment(getHandle());
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (Adjustment)obj;
 		return new Adjustment(hndl);
@@ -224,7 +227,6 @@ public class TreeView extends Container {
 	
 	/**
 	 * Returns true if the headers on the TreeView are visible.
-	 * @return
 	 */
 	public boolean getHeadersVisible() {
 		return gtk_tree_view_get_headers_visible(getHandle());
@@ -262,15 +264,15 @@ public class TreeView extends Container {
 
 	/**
 	 * Gets the column at the given position in the tree view.
-	 * @param n  The position of the column, counting from 0.
+	 * @param position  The position of the column, counting from 0.
 	 * @return The TreeViewColumn, or <code>null</code> if the position is outside the range 
 	 * of columns.
 	 */
 	public TreeViewColumn getColumn(int position) {
-		int h = gtk_tree_view_get_column(getHandle(), position);
-		if (h < 0)
+	    Handle h = gtk_tree_view_get_column(getHandle(), position);
+		if (h == null)
 			return null;
-		GObject obj = retrieveGObject(h);
+		GObject obj = getGObjectFromHandle(h);
 		if (null != obj)
 			return (TreeViewColumn)obj;
 		return new TreeViewColumn(h);
@@ -298,10 +300,10 @@ public class TreeView extends Container {
 	}
 	
 	public TreeViewColumn getExpanderColumn() {
-		int hndl = gtk_tree_view_get_expander_column(getHandle());
-		if (hndl < 0)
+	    Handle hndl = gtk_tree_view_get_expander_column(getHandle());
+		if (hndl == null)
 			return null;
-		GObject obj = retrieveGObject(hndl);
+		GObject obj = getGObjectFromHandle(hndl);
 		if (null != obj)
 			return (TreeViewColumn)obj;
 		return new TreeViewColumn(hndl);
@@ -313,11 +315,12 @@ public class TreeView extends Container {
 	 */
 
 	/**
-	 * Moves the alignments of the view to the position specified by column and 
-	 * path. . rowAlign determines where the row is placed, 
-	 * and colAlign determines where column is placed. Both are expected to be 
-	 * between 0.0 and 1.0. 0.0 means left/top alignment, 1.0 means 
-	 * right/bottom alignment, 0.5 means center.
+	 * Moves the alignments of the view to the position specified
+	 * by </tt>column</tt> and <tt>path</tt>. <tt>rowAlign</tt>
+	 * determines where the row is placed, and <tt>colAlign</tt>
+	 * determines where column is placed. Both are expected to be
+	 * between 0.0 and 1.0. 0.0 means left/top alignment, 1.0
+	 * means right/bottom alignment, 0.5 means center.
 	 *
 	 * @param path The path of the row to move to
 	 * @param column The TreeViewColumn to move horizontally to
@@ -356,7 +359,7 @@ public class TreeView extends Container {
 	 * @param column The TreeViewColumn to move horizontally to
 	 */
 	public void scrollToCell(TreeViewColumn column) {
-		gtk_tree_view_scroll_to_cell(getHandle(), 0, column.getHandle(), false, 0, 0);
+		gtk_tree_view_scroll_to_cell(getHandle(), null, column.getHandle(), false, 0, 0);
 	}
 
 	/**
@@ -365,7 +368,7 @@ public class TreeView extends Container {
 	 * @param path The path of the row to move to, or NULL.
 	 */
 	public void scrollToCell(TreePath path) {
-		gtk_tree_view_scroll_to_cell(getHandle(), path.getHandle(), 0, false, 0, 0);
+		gtk_tree_view_scroll_to_cell(getHandle(), path.getHandle(), null, false, 0, 0);
 	}
 
 	/**
@@ -388,23 +391,21 @@ public class TreeView extends Container {
 	 * Returns the current path
 	 */
 	public TreePath getCursorPath() {
-		int[] path = new int[1];
-		int[] column = new int[1];
-		gtk_tree_view_get_cursor(getHandle(), path, column);
-		return new TreePath(path[0]);
+		Handle path = GObject.getNullHandle();
+		gtk_tree_view_get_cursor(getHandle(), path, null);
+		return new TreePath(path);
 	}
 
 	/** 
 	 * Returns the current column 
 	 */
 	public TreeViewColumn getCursorColumn() {
-		int[] path = new int[1];
-		int[] column = new int[1];
-		gtk_tree_view_get_cursor(getHandle(), path, column);
-		GObject obj = retrieveGObject(column[0]);
+		Handle column = GObject.getNullHandle();
+		gtk_tree_view_get_cursor(getHandle(), null, column);
+		GObject obj = getGObjectFromHandle(column);
 		if (null != obj)
 			return (TreeViewColumn)obj;
-		return new TreeViewColumn(column[0]);
+		return new TreeViewColumn(column);
 	}
 
 	/**
@@ -508,12 +509,12 @@ public class TreeView extends Container {
 	 * Returns an array of all the columns currently in the view
 	 */
 	public TreeViewColumn[] getColumns(){
-		int[] hndls = gtk_tree_view_get_columns(getHandle());
+	    Handle[] hndls = gtk_tree_view_get_columns(getHandle());
 		if (null == hndls)
 			return null;
 		TreeViewColumn[] columns = new TreeViewColumn[hndls.length];
 		for (int i = 0; i < hndls.length; i++) {
-			GObject obj = retrieveGObject(hndls[i]);
+			GObject obj = getGObjectFromHandle(hndls[i]);
 			if (null != obj)
 				columns[i] = (TreeViewColumn)obj;
 			else
@@ -579,21 +580,43 @@ public class TreeView extends Container {
 		return gtk_tree_view_get_enable_search(getHandle());
 	}
 
+    /**
+     * @deprecated This method is broken and has been deprecated in favor of
+     * the {@link #setSearchDataColumn} method.
+     */
+    public void setSearchColumn(TreeViewColumn column) {
+        throw new RuntimeException( "This method is deprecated." +
+                                    "please use the method: "+
+                                    "TreeView#setSearchDataColumn(DataColumn)" );
+    }
+    /**
+     * @deprecated This method is broken and has been deprecated in favor of
+     * the {@link #getSearchDataColumn} method.
+     */
+    public TreeViewColumn getSearchColumn() {
+        throw new RuntimeException( "This method is deprecated." +
+                                    "please use the method: "+
+                                    "TreeView#getSearchDataColumn()" );
+    }
 	/**
 	 * Sets column as the column where the interactive search code should 
 	 * search in. Additionally, turns on interactive searching.
-	 * @param column The column to search in
+	 * @param column The column of the data model to search in.
 	 */
-	public void setSearchColumn(TreeViewColumn column) {
-		gtk_tree_view_set_search_column(getHandle(), column.getHandle());
+	public void setSearchDataColumn(DataColumn column) {
+		gtk_tree_view_set_search_column(getHandle(), column.getColumn());
 	}
 	
-	public TreeViewColumn getSearchColumn() {
-		int hndl = gtk_tree_view_get_search_column(getHandle());
-		GObject obj = retrieveGObject(hndl);
-		if (null != obj)
-			return (TreeViewColumn)obj;
-		return new TreeViewColumn(hndl);
+    /**
+     * Gets the column searched on by the interactive search code.
+     * @return A DataColumn object representing the column of the data model 
+     * in which the interactive search takes place.
+     */
+	public DataColumn getSearchDataColumn() {
+	    int datacol = gtk_tree_view_get_search_column(getHandle());
+            DataColumn dataColumn = new DataColumnInt();
+            dataColumn.setColumn( datacol );
+            return dataColumn;
 	}
 
 	/**
@@ -603,10 +626,96 @@ public class TreeView extends Container {
 		return new Type(gtk_tree_view_get_type());
 	}
 	
+    /**
+     * Enables or disables the fixed height mode of this TreeView. Fixed
+     * height mode speeds up TreeView by assuming that all rows
+     * have the same height. Only enable this option if all rows are
+     * the same height and all columns are of type
+     * {@link TreeViewColumnSizing#FIXED}.
+     *
+     * @param enable TRUE to enable fixed height mode.
+     */
+    public void setFixedHeightMode( boolean enable ) {
+        gtk_tree_view_set_fixed_height_mode( getHandle(), enable );
+    }
+    /**
+     * Returns whether fixed height mode is turned on for this TreeView.
+     */
+    public boolean getFixedHeightMode() {
+        return gtk_tree_view_get_fixed_height_mode( getHandle() );
+    }
+    /**
+     * Enables of disables the hover selection mode of this
+     * TreeView. Hover selection makes the selected row follow the
+     * pointer. Currently, this works only for the selection modes
+     * {@link SelectionMode#SINGLE} and {@link SelectionMode#BROWSE}.
+     *
+     * @param hover TRUE to enable hover selection mode.
+     */
+    public void setHoverSelection( boolean hover ) {
+        gtk_tree_view_set_hover_selection( getHandle(), hover );
+    }
+    /**
+     * Returns whether hover selection mode is turned on for this TreeView.
+     */
+    public boolean getHoverSelection() {
+        return gtk_tree_view_get_hover_selection( getHandle() );
+    }
+    /**
+     * Enables of disables the hover expansion mode of this
+     * TreeView. Hover expansion makes rows expand or collaps if the
+     * pointer moves over them.
+     *
+     * @param expand TRUE to enable hover selection mode.
+     */
+    public void setHoverExpand( boolean expand ) {
+        gtk_tree_view_set_hover_expand( getHandle(), expand );
+    }
+    /**
+     * Returns whether hover expansion mode is turned on for TreeView.
+     */
+    public boolean getHoverExpand() {
+        return gtk_tree_view_get_hover_expand( getHandle() );
+    }
 
-	/***************************************
-	 * Event Handler Related code
-	 ****************************************/
+    /**
+     * Set the object which is used to determine whether a row should be drawn
+     * as a separator.
+     *
+     * @param method The <tt>TreeViewRowSeparatorMethod</tt> or
+     * <tt>null</tt> if no separators are to be drawn (the default).
+     */
+    public void setRowSeparatorMethod( TreeViewRowSeparatorMethod method ) {
+        separatorFunc = method;
+        if ( method == null ) {
+            gtk_tree_view_set_row_separator_func( getHandle(), null,
+                                                  "handleSeparatorFunc" );
+        } else {
+            gtk_tree_view_set_row_separator_func( getHandle(), this,
+                                                  "handleSeparatorFunc" );
+        }
+    }
+
+    protected boolean handleSeparatorFunc( Handle model, Handle iter ) {
+        TreeModel mod = (TreeModel)getGObjectFromHandle(model);
+        TreeIter it = new TreeIter( iter, mod );
+        return separatorFunc.isSeparator( mod, it );
+    }
+
+    /**
+     * Get the object which is used to determine whether a row should be
+     * drawn as a separator.
+     *
+     * @return The <tt>TreeViewRowSeparatorMethod</tt> or <tt>null</tt>
+     * if no method has been set.
+     */
+    public TreeViewRowSeparatorMethod getRowSeparatorMethod() {
+        return separatorFunc;
+    }
+
+    //
+    // Event Handler Related code
+    //
 
 	/**
 	 * Listeners for handling toggle events
@@ -626,7 +735,7 @@ public class TreeView extends Container {
 				evtMap.initialize(this, TreeViewEvent.Type.EXPAND_COLLAPSE_CURSOR_ROW);
 				evtMap.initialize(this, TreeViewEvent.Type.ROW_ACTIVATED);
 				evtMap.initialize(this, TreeViewEvent.Type.ROW_COLLAPSED);
-				evtMap.initialize(this, TreeViewEvent.Type.ROW_EXPANDED);
+				evtMap.initialize(this, TreeViewEvent.Type.ROW_COLLAPSED);
 				evtMap.initialize(this, TreeViewEvent.Type.SELECT_ALL);
 				evtMap.initialize(this, TreeViewEvent.Type.SELECT_CURSOR_PARENT);
 				evtMap.initialize(this, TreeViewEvent.Type.SELECT_CURSOR_ROW);
@@ -690,10 +799,10 @@ public class TreeView extends Container {
 		}
 	}
 
-	private void handleRowActivated(int path, int column) {
+	private void handleRowActivated(Handle path, Handle column) {
 		TreeViewEvent event = new TreeViewEvent(this, TreeViewEvent.Type.ROW_ACTIVATED);
 		event.setTreePath(new TreePath(path));
-		GObject obj = retrieveGObject(column);
+		GObject obj = getGObjectFromHandle(column);
 		if (null != obj)
 			event.setTreeColumn((TreeViewColumn)obj);
 		else
@@ -701,14 +810,14 @@ public class TreeView extends Container {
 		fireTreeViewEvent(event);
 	}
 
-	private void handleRowExpanded(int iter, int path) {
+	private void handleRowExpanded(Handle iter, Handle path) {
 		TreeViewEvent event = new TreeViewEvent(this, TreeViewEvent.Type.ROW_EXPANDED);
 		event.setTreePath(new TreePath(path));
 		event.setTreeIter(new TreeIter(iter, getModel()));
 		fireTreeViewEvent(event);
 	}
 
-	private void handleRowCollapsed(int iter, int path) {
+	private void handleRowCollapsed(Handle iter, Handle path) {
 		TreeViewEvent event = new TreeViewEvent(this, TreeViewEvent.Type.ROW_COLLAPSED);
 		event.setTreePath(new TreePath(path));
 		event.setTreeIter(new TreeIter(iter, getModel()));
@@ -772,38 +881,12 @@ public class TreeView extends Container {
 		return cls;
 	}
 
-	public GtkEventType getEventType(String signal) {
-		GtkEventType et = evtMap.getEventType(signal);
+	public EventType getEventType(String signal) {
+		EventType et = evtMap.getEventType(signal);
 		if (et == null) et = super.getEventType(signal);
 		return et;
 	}
 
-	public PosInfo getPathAtPos(int x, int y) {
-	    int[] path = new int[1];
-	    int[] column = new int[1];
-	    int[] cellX = new int[1];
-	    int[] cellY = new int[1];
-	    if (gtk_tree_view_get_path_at_pos(getHandle(),x,y,path,column,cellX,cellY)) {
-	        PosInfo pos = new PosInfo(new TreePath(path[0]), new TreeViewColumn(column[0]), 
-	                cellX[0], cellY[0]);
-	        return pos;
-	    } else
-	        return null;
-	}
-	
-	public static class PosInfo {
-	    public int cellX;
-	    public int cellY;
-	    public TreePath path;
-	    public TreeViewColumn column;
-	    public PosInfo(TreePath path, TreeViewColumn column, int x, int y) {
-	        this.path = path;
-	        this.column = column;
-	        this.cellX = x;
-	        this.cellY = y;
-	    }
-	}
-	
 	private static EventMap evtMap = new EventMap();
 	static {
 		addEvents(evtMap);
@@ -823,55 +906,52 @@ public class TreeView extends Container {
 		anEvtMap.addEvent("select_cursor_row", "handleSelectCursorRow", TreeViewEvent.Type.SELECT_CURSOR_ROW, TreeViewListener.class);
 		anEvtMap.addEvent("toggle_cursor_row", "handleToggleCursorRow", TreeViewEvent.Type.TOGGLE_CURSOR_ROW, TreeViewListener.class);
 		anEvtMap.addEvent("expand_collapse_cursor_row", "handleExpandCollapseCursorRow", TreeViewEvent.Type.EXPAND_COLLAPSE_CURSOR_ROW, TreeViewListener.class);
-		anEvtMap.addEvent("sel	ect_cursor_parent", "handleSelectCursorParent", TreeViewEvent.Type.SELECT_CURSOR_PARENT, TreeViewListener.class);
+		anEvtMap.addEvent("select_cursor_parent", "handleSelectCursorParent", TreeViewEvent.Type.SELECT_CURSOR_PARENT, TreeViewListener.class);
 		anEvtMap.addEvent("start_interactive_search", "handleStartInteractiveSearch", TreeViewEvent.Type.START_INTERACTIVE_SEARCH, TreeViewListener.class);
 	}
 
 
-	/****************************************
-	 * BEGINNING OF JNI CODE
-	 ****************************************/
 	native static final protected int gtk_tree_view_get_type();
-	native static final protected int gtk_tree_view_new();
-	native static final protected int gtk_tree_view_new_with_model(int model);
-	native static final protected int gtk_tree_view_get_model(int treeView);
-	native static final protected void gtk_tree_view_set_model(int treeView, int model);
-	native static final protected int gtk_tree_view_get_selection(int treeView);
-	native static final protected int gtk_tree_view_get_hadjustment(int treeView);
-	native static final protected void gtk_tree_view_set_hadjustment(int treeView, int adjustment);
-	native static final protected int gtk_tree_view_get_vadjustment(int treeView);
-	native static final protected void gtk_tree_view_set_vadjustment(int treeView, int adjustment);
-	native static final protected boolean gtk_tree_view_get_headers_visible(int treeView);
-	native static final protected void gtk_tree_view_set_headers_visible(int treeView, boolean headersVisible);
-	native static final protected void gtk_tree_view_columns_autosize(int treeView);
-	native static final protected void gtk_tree_view_set_headers_clickable(int treeView, boolean setting);
-	native static final protected void gtk_tree_view_set_rules_hint(int treeView, boolean setting);
-	native static final protected boolean gtk_tree_view_get_rules_hint(int treeView);
-	native static final protected int gtk_tree_view_append_column(int treeView, int column);
-	native static final protected int gtk_tree_view_remove_column(int treeView, int column);
-	native static final protected int gtk_tree_view_insert_column(int treeView, int column, int position);
-	native static final protected int gtk_tree_view_get_column(int treeView, int n);
-	native static final protected int[] gtk_tree_view_get_columns(int treeView);
-	native static final protected void gtk_tree_view_move_column_after(int treeView, int column, int baseColumn);
-	native static final protected void gtk_tree_view_set_expander_column(int treeView, int column);
-	native static final protected int gtk_tree_view_get_expander_column(int treeView);
-	native static final protected void gtk_tree_view_scroll_to_point(int treeView, int treeX, int treeY);
-	native static final protected void gtk_tree_view_scroll_to_cell(int treeView, int path, int column, boolean useAlign, double rowAlign, double colAlign);
-	native static final protected void gtk_tree_view_row_activated(int treeView, int path, int column);
-	native static final protected void gtk_tree_view_expand_all(int treeView);
-	native static final protected void gtk_tree_view_collapse_all(int treeView);
-	native static final protected void gtk_tree_view_expand_to_path(int treeView, int path);
-	native static final protected boolean gtk_tree_view_expand_row(int treeView, int path, boolean openAll);
-	native static final protected boolean gtk_tree_view_collapse_row(int treeView, int path);
-	native static final protected boolean gtk_tree_view_row_expanded(int treeView, int path);
-	native static final protected void gtk_tree_view_set_reorderable(int treeView, boolean reorderable);
-	native static final protected boolean gtk_tree_view_get_reorderable(int treeView);
-	native static final protected void gtk_tree_view_set_cursor(int treeView, int path, int focusColumn, boolean startEditing);
-	native static final protected void gtk_tree_view_set_cursor_on_cell(int treeView, int path, int focusColumn, int focusCell, boolean startEditin);
-	native static final protected void gtk_tree_view_get_cursor(int treeView, int[] path, int[] focusColumn);
-//	native static final protected int gtk_tree_view_get_bin_window(int treeView);
-	native static final protected boolean gtk_tree_view_get_path_at_pos(int treeView, int x, int y, int[] path, int[] column, int[] cellX, int[] cellY);
-//	native static final protected int gtk_tree_view_get_cell_area(int treeView, int path, int column);
+	native static final protected Handle gtk_tree_view_new();
+	native static final protected Handle gtk_tree_view_new_with_model(Handle model);
+	native static final protected Handle gtk_tree_view_get_model(Handle treeView);
+	native static final protected void gtk_tree_view_set_model(Handle treeView, Handle model);
+	native static final protected Handle gtk_tree_view_get_selection(Handle treeView);
+	native static final protected Handle gtk_tree_view_get_hadjustment(Handle treeView);
+	native static final protected void gtk_tree_view_set_hadjustment(Handle treeView, Handle adjustment);
+	native static final protected Handle gtk_tree_view_get_vadjustment(Handle treeView);
+	native static final protected void gtk_tree_view_set_vadjustment(Handle treeView, Handle adjustment);
+	native static final protected boolean gtk_tree_view_get_headers_visible(Handle treeView);
+	native static final protected void gtk_tree_view_set_headers_visible(Handle treeView, boolean headersVisible);
+	native static final protected void gtk_tree_view_columns_autosize(Handle treeView);
+	native static final protected void gtk_tree_view_set_headers_clickable(Handle treeView, boolean setting);
+	native static final protected void gtk_tree_view_set_rules_hint(Handle treeView, boolean setting);
+	native static final protected boolean gtk_tree_view_get_rules_hint(Handle treeView);
+	native static final protected int gtk_tree_view_append_column(Handle treeView, Handle column);
+	native static final protected int gtk_tree_view_remove_column(Handle treeView, Handle column);
+	native static final protected int gtk_tree_view_insert_column(Handle treeView, Handle column, int position);
+	native static final protected Handle gtk_tree_view_get_column(Handle treeView, int n);
+	native static final protected Handle[] gtk_tree_view_get_columns(Handle treeView);
+	native static final protected void gtk_tree_view_move_column_after(Handle treeView, Handle column, Handle baseColumn);
+	native static final protected void gtk_tree_view_set_expander_column(Handle treeView, Handle column);
+	native static final protected Handle gtk_tree_view_get_expander_column(Handle treeView);
+	native static final protected void gtk_tree_view_scroll_to_point(Handle treeView, int treeX, int treeY);
+	native static final protected void gtk_tree_view_scroll_to_cell(Handle treeView, Handle path, Handle column, boolean useAlign, double rowAlign, double colAlign);
+	native static final protected void gtk_tree_view_row_activated(Handle treeView, Handle path, Handle column);
+	native static final protected void gtk_tree_view_expand_all(Handle treeView);
+	native static final protected void gtk_tree_view_collapse_all(Handle treeView);
+	native static final protected void gtk_tree_view_expand_to_path(Handle treeView, Handle path);
+	native static final protected boolean gtk_tree_view_expand_row(Handle treeView, Handle path, boolean openAll);
+	native static final protected boolean gtk_tree_view_collapse_row(Handle treeView, Handle path);
+	native static final protected boolean gtk_tree_view_row_expanded(Handle treeView, Handle path);
+	native static final protected void gtk_tree_view_set_reorderable(Handle treeView, boolean reorderable);
+	native static final protected boolean gtk_tree_view_get_reorderable(Handle treeView);
+	native static final protected void gtk_tree_view_set_cursor(Handle treeView, Handle path, Handle focusColumn, boolean startEditing);
+	native static final protected void gtk_tree_view_set_cursor_on_cell(Handle treeView, Handle path, Handle focusColumn, Handle focusCell, boolean startEditin);
+	native static final protected void gtk_tree_view_get_cursor(Handle treeView, Handle path, Handle focusColumn);
+	native static final protected Handle gtk_tree_view_get_bin_window(Handle treeView);
+	native static final protected boolean gtk_tree_view_get_path_at_pos(Handle treeView, int x, int y, Handle path, Handle column, int cellX, int cellY);
+    //	native static final protected int gtk_tree_view_get_cell_area(int treeView, int path, int column);
 //	native static final protected int gtk_tree_view_get_background_area(int treeView, int path, int column);
 //	native static final protected int gtk_tree_view_get_visible_rect(int treeView);
 //	native static final protected void gtk_tree_view_widget_to_tree_coords(int treeView, int wx, int wy, int[] tx, int[] ty);
@@ -881,11 +961,19 @@ public class TreeView extends Container {
 //	native static final protected void gtk_tree_view_unset_rows_drag_source(int treeView);
 //	native static final protected void gtk_tree_view_unset_rows_drag_dest(int treeView);
 //	native static final protected int gtk_tree_view_create_row_drag_icon(int treeView, int path);
-	native static final protected void gtk_tree_view_set_enable_search(int treeView, boolean enableSearch);
-	native static final protected boolean gtk_tree_view_get_enable_search(int treeView);
-	native static final protected int gtk_tree_view_get_search_column(int treeView);
-	native static final protected void gtk_tree_view_set_search_column(int treeView, int column);
-	/****************************************
-	 * END OF JNI CODE
-	 ****************************************/
+	native static final protected void gtk_tree_view_set_enable_search(Handle treeView, boolean enableSearch);
+	native static final protected boolean gtk_tree_view_get_enable_search(Handle treeView);
+	native static final protected int gtk_tree_view_get_search_column(Handle treeView);
+	native static final protected void gtk_tree_view_set_search_column(Handle treeView, int dataColumn);
+
+    // GTK 2.6 additions.
+    native static final private void gtk_tree_view_set_fixed_height_mode(Handle tree_view, boolean enable);
+    native static final private boolean gtk_tree_view_get_fixed_height_mode(Handle tree_view);
+    native static final private void gtk_tree_view_set_hover_selection(Handle tree_view, boolean hover);
+    native static final private boolean gtk_tree_view_get_hover_selection(Handle tree_view);
+    native static final private void gtk_tree_view_set_hover_expand(Handle tree_view, boolean expand);
+    native static final private boolean gtk_tree_view_get_hover_expand(Handle tree_view);
+    native static final private void gtk_tree_view_set_row_separator_func(Handle tree_view, TreeView tview, String callback );
+    //native static final private int gtk_tree_view_get_row_separator_func(Handle tree_view);
+
 }
